/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.adb;

// Import the ADB loader to ensure that the libs get loaded
import com.tplan.adb.AdbLoader;

import com.android.ddmlib.IDevice;
import com.android.ddmlib.ShellCommandUnresponsiveException;
import com.tplan.robot.gui.MainFrame;
import com.tplan.robot.gui.components.FileChooserManager;
import com.tplan.robot.gui.components.StringComponent;
import com.tplan.robot.preferences.Preference;
import com.tplan.robot.preferences.PreferenceComponentManager;
import com.tplan.robot.remoteclient.RemoteDesktopClient;
import com.tplan.robot.remoteclient.adb.AdbClient;
import com.tplan.robot.scripting.DefaultJavaTestScript;
import com.tplan.robot.scripting.ParametrizedTestScript;
import com.tplan.robot.scripting.StopRequestException;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import javax.swing.JFileChooser;

/**
 * Pull (copy) a file or directory from the Android device to the local file
 * system.
 */
public class Pull extends DefaultJavaTestScript implements ParametrizedTestScript,
        Runnable, PreferenceComponentManager, ActionListener, FileChooserManager, Constants {

    // Parameter "local"
    public static final String PARAM_LOCAL = "local";
    // Parameter "remote"
    public static final String PARAM_REMOTE = "remote";
    //
    // Member variables populated per execution
    protected IDevice device;
    protected String local;
    protected String remote;
    protected Exception exception;
    private StringComponent remoteFileComponent;

    @Override
    public void test() {
        int exitCode = 1;
        try {
            // Load the command parameters - the locat and remote files
            // and the optional timeout
            final String local = getVariableAsString(PARAM_LOCAL);
            final String remote = getVariableAsString(PARAM_REMOTE);

            // There must be a connected "Android over ADB" client
            RemoteDesktopClient client = getContext().getClient();
            if (client != null && client.isConnected() && client instanceof AdbClient) {
                AdbClient cl = (AdbClient) client;
                IDevice dev = (IDevice) cl.getIDevice();
                if (dev != null) {
                    try {
                        // Populate the member variables
                        this.device = dev;
                        this.local = local;
                        this.remote = remote;
                        this.exception = null;

                        // Start the operation in a thread so that we can
                        // continue in case that the user stops the script.
                        // As the IDevice API doesn't allow to stop the operation
                        // we will just let the thread run on its own.
                        Thread t = new Thread(this);
                        t.start();

                        // Wait until the thread finishes. If the stop is requested
                        // cancel it.
                        while (t.isAlive()) {
                            if (getContext().getMasterInterpret().isStop()) {
                                t.interrupt();
                                throw new StopRequestException();
                            }
                            Thread.sleep(5);
                        }

                        // If the command executed in the thread threw an exception
                        // then rethrow it
                        if (exception != null) {
                            throw exception;
                        }

                        exitCode = 0;
                    } catch (Exception ex) {
                        if (ex instanceof ShellCommandUnresponsiveException || ex instanceof TimeoutException) {
                            log("Failed to pull file " + remote + " from the Android device: Operation timed out.", Level.SEVERE);
                        } else if (ex instanceof StopRequestException) {
                            log("The script was stopped while pulling file " + remote + " from the Android device to " + local + ". The result of the operation is not known.", Level.WARNING);
                        } else {
                            log("Failed to pull file " + remote + " from the Android device: Operation failed with " + ex.getClass().getName() + ": " + ex.getMessage(), Level.SEVERE);
                        }
                    }
                }
            } else {
                if (client == null || !client.isConnected()) {
                    log("Failed to pull file " + remote + " because no Android device is connected!", Level.SEVERE);
                } else {
                    log("Failed to pull file " + remote + " from the Android device because the current connection ("
                            + client.getConnectString() + ") is not an Android Over ADB one!", Level.SEVERE);
                }
            }
        } catch (StopRequestException ex) {
            throw ex;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            getContext().setVariable("_EXIT_CODE", exitCode);
        }
    }

    public List<String> getParameters() {
        return Arrays.asList(new String[]{PARAM_LOCAL, PARAM_REMOTE});
    }

    public List getParameterValues(String parameterName) {
        return null;
    }

    public void setParameters(Map<String, String> paramsAndValues) {
    }

    public String getLongDescription() {
        return getShortDescription();
    }

    public String getShortDescription() {
        return "Pull (copy) a file from the device to the local computer over the ADB bridge.";
    }

    public void run() {
        try {
            String target = local;
            File localFile = new File(local);

            // This ensures that the command behaves a bit more user friendly
            // than ADB which rejects commands like "pull /sdcard/foo.txt C:\temp".
            // If the local file exists and it is a directory we have to update
            // the target to something like "C:\temp\foo.txt".
            if (localFile.exists() && localFile.isDirectory()) {
                int i = remote.lastIndexOf('/');
                if (i >= 0) {
                    target = target + File.separator + remote.substring(i + 1);
                }
            }
            device.pullFile(remote, target);
        } catch (Exception ex) {
//            ex.printStackTrace();
            this.exception = ex;
        }
    }

    public List<Preference> getVisualParameters() {
        List<Preference> l = new ArrayList<Preference>();
        String title = "ADB Pull File";
        String desc = "<html><body><a href=\""
                + ADB_FILE_HELP_URL + "\">Pull (copy)</a> the specified remote file or directory "
                + "from the Android device connected<br/>over the "
                + "<a href=\"" + ANDROID_OVER_ADB_URL + "\">Android Over ADB</a> connection "
                + "to the local file system."
                + "<br/>For details see the <a href=\"" + PLUGIN_DOC_URL + "#" + getClass().getSimpleName().toLowerCase()
                + "\">" + getClass().getSimpleName() + " specification</a>."
                + "</body></html>";
        Preference p = new Preference(PARAM_REMOTE, Preference.TYPE_STRING_WITH_ACTION, "Remote file/dir", desc);
        p.setComponentManager(this);
        p.setPreferredContainerName(title);
        p.setAcceptEmptyValue(false);
        p.setMandatory(true);
        p.setHint(title,
                "<html><body>"
                + "<p>Provide the full path of the target file (directory) in the Android file system. "
                + "You may use the button on the right to browse the file system of the currectly "
                + "connected Android device.</p>"
                + "</body></html>"
                );
        l.add(p);

        p = new Preference(PARAM_LOCAL, Preference.TYPE_FILE, "Local file/dir", null);
        p.setPreferredContainerName(title);
        p.setAcceptEmptyValue(false);
        p.setFileChooserManager(this);
        p.setMandatory(true);
        p.setHint(title,
                "<html><body>"
                + "<p>Provide the full path of the source local file (directory). To specify the file relatively "
                + "to one of the standard paths use one of the default path variables such as:</p>"
                + "<ul><li><code>_INSTALL_DIR</code> - the Robot installation directory,</li>"
                + "<li><code>_PROJECT_DIR</code> - home folder of the project the script belongs to,</li>"
                + "<li><code>_SCRIPT_DIR</code> - the script folder (TPR scripts only).</li>"
                + "</ul><p>For example, to copy the <code>data.txt</code> file stored in the project home folder "
                + "enter:</p><br/>"
                + "<ul><code>\"{_PROJECT_DIR}" + File.separator + "data.txt\"</code></ul>"
                + "</body></html>"
                );
        l.add(p);
        return l;
    }

    public void setFileChooser(JFileChooser chooser) {
        chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
    }

    public void componentCreated(Preference p, Object component, Object container) {
        if (component instanceof StringComponent) {
            remoteFileComponent = (StringComponent) component;
            StringComponent sc = (StringComponent) component;
            sc.getButton().addActionListener(this);
            boolean enabled = false;
            RemoteDesktopClient client = MainFrame.getInstance().getClient();
            if (client != null && client.isConnected() && client instanceof AdbClient) {
                AdbClient cl = (AdbClient) client;
                IDevice dev = (IDevice) cl.getIDevice();
                if (dev != null) {
                    enabled = true;
                }
            }
            sc.getButton().setEnabled(enabled);
        }
    }

    public void goingToLoad(Preference p, Object component, Object container, Object configuration) {
    }

    public void savePerformed(Preference p, Object component, Object container, Object configuration) {
    }

    public void actionPerformed(ActionEvent e) {
        RemoteDesktopClient client = MainFrame.getInstance().getClient();
        if (client != null && client.isConnected() && client instanceof AdbClient) {
            AdbClient cl = (AdbClient) client;
            IDevice dev = (IDevice) cl.getIDevice();
            if (dev != null) {
                Component owner = (Component) e.getSource();
                AndroidFileViewer dlg = new AndroidFileViewer(owner, dev);
                String selected = remoteFileComponent.getText();
                if (selected != null && !selected.isEmpty()) {
                    dlg.setSelectedFile(selected);
                }
                dlg.setVisible(true);
                String s = dlg.getSelectedFile();
                if (s != null) {
                    remoteFileComponent.setText(s);
                }
            }
        }
    }
}
