/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.adb;

// Import the ADB loader to ensure that the libs get loaded
import com.tplan.adb.AdbLoader;

import com.android.ddmlib.CollectingOutputReceiver;
import com.android.ddmlib.IDevice;
import com.android.ddmlib.ShellCommandUnresponsiveException;
import com.android.ddmlib.TimeoutException;
import com.tplan.robot.preferences.Preference;
import com.tplan.robot.scripting.*;
import com.tplan.robot.remoteclient.RemoteDesktopClient;
import com.tplan.robot.remoteclient.adb.AdbClient;
import com.tplan.robot.util.Utils;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import static com.tplan.adb.Shell.VAR_SHELL_OUTPUT;
import java.util.concurrent.TimeUnit;

/**
 * Start an application identified by the package and activity on the connected
 * Android device. Available since version 2.
 * @since 2.0
 */
public class Start extends DefaultJavaTestScript
        implements ParametrizedTestScript, Runnable, Constants {

    // Parameter "command"
    public static final String PARAM_ACTIVITY = "activity";
    // Parameter "log"
    public static final String PARAM_LOG = "log";
    // Parameter "timeout"
    public static final String PARAM_TIMEOUT = "timeout";
    //
    // Member variables populated per execution
    IDevice device;
    String command;
    CollectingOutputReceiver receiver;
    Integer timeout;
    Exception exception;

    protected String createShellCommand() {
        // Load the command parameters - the command, the "record to the log" flag
        // and the optional timeout
        String activity = getVariableAsString(PARAM_ACTIVITY);
        if (activity == null) {
            String pkg = (String) getContext().get(Install.VAR_PACKAGE);
            String act = (String) getContext().get(Install.VAR_ACTIVITY);
            if (pkg != null && act != null) {
                activity = pkg + "/" + act;
            } else if (getContext().getMasterInterpret().isPartiallyExecuting() && Install.lastActivity != null && Install.lastPackage != null) {
                activity = Install.lastPackage + "/" + Install.lastActivity;
            }
        }
        if (activity == null) {
            throw new IllegalArgumentException("No activity name was specified and "
                    + "no application has been recently installed through the " + Install.class.getName() + " script!");
        }
        return "am start -n " + activity;
    }

    @Override
    public void test() {
        int exitCode = 1;
        try {
            // Clear the shell output variable
            getContext().getVariables().remove(VAR_SHELL_OUTPUT);

            Boolean b = getVariableAsBoolean(PARAM_LOG);
            boolean logResult = b != null && b;

            String stimeout = getVariableAsString(PARAM_TIMEOUT);
            if (stimeout != null && !stimeout.trim().isEmpty()) {
                timeout = getContext().getParser().parseTime(stimeout, PARAM_TIMEOUT).intValue();
            } else {
                timeout = null;
            }

            String command = createShellCommand();

            // There must be a connected "Android over ADB" client
            RemoteDesktopClient client = getContext().getClient();
            if (client != null && client.isConnected() && client instanceof AdbClient) {
                AdbClient cl = (AdbClient) client;
                IDevice dev = (IDevice) cl.getIDevice();
                if (dev != null) {
                    try {
                        // Populate the member variables
                        this.receiver = new CollectingOutputReceiver();
                        this.device = dev;
                        this.command = command;
                        this.exception = null;

                        // Start the shell command in a thread so that we can
                        // kill it in case that the user stops the script
                        Thread t = new Thread(this);
                        t.start();

                        // Wait until the thread finishes. If the stop is requested
                        // cancel it.
                        while (t.isAlive() && !receiver.isCancelled()) {
                            if (getContext().getMasterInterpret().isStop()) {
                                receiver.cancel();
                                break;
                            }
                            Thread.sleep(5);
                        }

                        // If the command executed in the thread threw an exception 
                        // rethrow it
                        if (exception != null) {
                            throw exception;
                        }

                        // If the thread finished populate the output text variable
                        // and eventually record it to the execution log
                        if (!receiver.isCancelled()) {
                            String output = receiver.getOutput();
                            getContext().setVariable(VAR_SHELL_OUTPUT, output);
                            if (logResult) {
                                output = output.replace("\n", "<br/>");
                                log("The shell command of '" + command + "' produced:\n" + output, Level.INFO);
                            }
                        }
                        exitCode = 0;
                    } catch (Exception ex) {
                        if (ex instanceof ShellCommandUnresponsiveException || ex instanceof TimeoutException) {
                            log("The shell command of '" + command + "' timed out (timeout=" + Utils.getTimePeriodForDisplay(timeout, false, true) + ")", Level.SEVERE);
                            exitCode = 2;
                        } else {
                            log("The shell command of '" + command + "' failed with " + ex.getClass().getName() + ": " + ex.getMessage(), Level.SEVERE);
                        }
                    }
                }
            } else {
                if (client == null || !client.isConnected()) {
                    log("The shell command of '" + command + "' failed because no Android device is connected!", Level.SEVERE);
                } else {
                    log("The shell command of '" + command + "' failed because the current connection ("
                            + client.getConnectString() + ") is not an Android Over ADB one!", Level.SEVERE);
                }
            }
        } catch (StopRequestException ex) {
            throw ex;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            getContext().setVariable("_EXIT_CODE", exitCode);
        }
    }

    public List<String> getParameters() {
        return Arrays.asList(new String[]{PARAM_ACTIVITY, PARAM_LOG, PARAM_TIMEOUT});
    }

    public List getParameterValues(String parameterName) {
        if (PARAM_LOG.equals(parameterName)) {
            return Arrays.asList(new String[]{"true", "false"});
        }
        return null;
    }

    public void setParameters(Map<String, String> paramsAndValues) {
    }

    public List<Preference> getVisualParameters() {
        List<Preference> l = new ArrayList<Preference>();
        String title = "Start Command";

        String desc = "<html><body>"
                + "Start an activity (an application) on the Android device connected over<br/>"
                + "the <a href=\"" + ANDROID_OVER_ADB_URL + "\">Android Over ADB</a> connection. The command calls internally<br/>"
                + "the Activity Manager <code>am start -n &lt;activity&gt;</code> shell command on the device. If the activity name<br/>"
                + "is omitted the command will start the last application installed through<br/>"
                + "the <code>com.tplan.adb.Install</code> script.<br/>"
                + "The shell command output will be stored to the <code>" + VAR_SHELL_OUTPUT + "</code> variable.<br/>"
                + " For details see the <a href=\"" + PLUGIN_DOC_URL + "#" + getClass().getSimpleName().toLowerCase()
                + "\">" + getClass().getSimpleName() + " specification</a>."
                + "</body></html>";

        Preference p = new Preference(PARAM_ACTIVITY, Preference.TYPE_STRING, "Activity (optional)", desc);
        p.setPreferredContainerName(title);
        p.setAcceptEmptyValue(false);
        p.setMandatory(false);
        p.setHint(title,
                "<html><body><p>Specify the activity name (optional). It has the format of:"
                + "<ul><code>[package]/[activity_class]</code></ul>"
                + "where <code>package</code> is the application package name and "
                + "<code>activity_class</code> is the name of the main (default) activity class. "
                + "The latter may be specified through a full name or a simple class name through the dot notation. "
                + "For example, the following notation is equivalent:"
                + "<ul>com.mycompany.myapp/com.mycompany.myapp.MyStartActivity<br/><br/>"
                + "com.mycompany.myapp/.MyStartActivity</ul>"
                + "For details see the <a href=\""
                + ADB_SHELL_HELP_URL + "\"><code>am start</code> shell command</a>.</p>"
                + "</body></html>");
        l.add(p);

        p = new Preference(PARAM_TIMEOUT, Preference.TYPE_TIME_INTERVAL, "Timeout (zero sets off)", null);
        p.setMinValue(0);
        p.setDefaultValue(0);
        p.setPreferredContainerName(title);
        p.setHint(title,
                "<html><body><p>The optional <code>timeout</code> parameter will make the command "
                + "time out and finish with the exit code of 2 if it lasts longer than the specified time value. "
                + "To disable the timeout leave the value on zero.</p></html>");
        l.add(p);

        p = new Preference(PARAM_LOG, Preference.TYPE_BOOLEAN, "Record the output to the execution log", null);
        p.setPreferredContainerName(title);
        p.setDefaultValue(false);
        p.setHint(title,
                "<html><body><p>The optional <code>log</code> parameter allows to write "
                + "the command output to the <a href=\"" + EXEC_LOG_HELP_URL + "\">execution log</a>. "
                + "It has the same effect as if you execute the <b>Log</b> command after the shell one as follows:</p><br>"
                + "<a href=\"" + TPR_SCRIPT_HELP_URL + "#log\">TPR test scripts:</a>"
                + "<ul><code>Log \"{" + VAR_SHELL_OUTPUT + "}\"</code></ul>"
                + "<a href=\"" + JAVA_SCRIPT_HELP_URL + "#log\">Java test scripts:</a>"
                + "<ul><code>log(getVariableAsString(\"" + VAR_SHELL_OUTPUT + "\"), java.util.logging.Level.INFO);</code></ul>"
                + "</html>");
        l.add(p);
        return l;
    }

    public String getShortDescription() {
        return "Start an activity on the Android device connected over the ADB connection.";
    }

    public String getLongDescription() {
        return getShortDescription();
    }

    public void run() {
        try {
            if (timeout == null || timeout == 0) {
                device.executeShellCommand(command, receiver);
            } else {
                device.executeShellCommand(command, receiver, timeout, TimeUnit.MILLISECONDS);
            }
        } catch (Exception ex) {
            this.exception = ex;
        }
    }
}
