/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.adb;

// Import the ADB loader to ensure that the libs get loaded
import com.tplan.adb.AdbLoader;

import static com.tplan.adb.Constants.ANDROID_OVER_ADB_URL;
import static com.tplan.adb.Shell.VAR_SHELL_OUTPUT;
import static com.tplan.adb.Start.PARAM_LOG;
import com.tplan.robot.preferences.Preference;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Kill all processes associated with the specified application package name. 
 * Available since version 2.
 * @since 2.0
 */
public class Stop extends Start {

    public static final String PARAM_PACKAGE = "package";
    public static final String PARAM_FORCE = "force";

    @Override
    protected String createShellCommand() {
        // Load the command parameters - the package, the "record to the log" flag
        // and the optional force flag
        String pkg = getVariableAsString(PARAM_PACKAGE);
        if (pkg == null) {
            pkg = (String) getContext().get(Install.VAR_PACKAGE);
            if (pkg != null) {
                // OK
            } else if (getContext().getMasterInterpret().isPartiallyExecuting() && Install.lastPackage != null) {
                pkg = Install.lastPackage;
            }
        }
        if (pkg == null) {
            throw new IllegalArgumentException("No package name was specified and "
                    + "no application has been recently installed through the " + Install.class.getName() + " script!");
        }
        Boolean force = getVariableAsBoolean(PARAM_FORCE);
        String cmd = force == null || force ? "force-stop" : "kill";
        return "am " + cmd + " " + pkg;
    }

    public List<String> getParameters() {
        return Arrays.asList(new String[]{PARAM_PACKAGE, PARAM_FORCE, PARAM_LOG});
    }

    public List<Preference> getVisualParameters() {
        List<Preference> l = new ArrayList<Preference>();
        String title = "Stop Command";

        String desc = "<html><body>"
                + "Stop/kill an application on the Android device connected over<br/>"
                + "the <a href=\"" + ANDROID_OVER_ADB_URL + "\">Android Over ADB</a> connection. The command calls internally<br/>"
                + "the Activity Manager <code>am force-stop &lt;package&gt;</code> shell command on the device. If the package name<br/>"
                + "is omitted the command will stop the last application installed through<br/>"
                + "the <code>com.tplan.adb.Install</code> script.<br/>"
                + "The shell command output will be stored to the <code>" + VAR_SHELL_OUTPUT + "</code> variable.<br/>"
                + " For details see the <a href=\"" + PLUGIN_DOC_URL + "#" + getClass().getSimpleName().toLowerCase()
                + "\">" + getClass().getSimpleName() + " specification</a>."
                + "</body></html>";

        Preference p = new Preference(PARAM_PACKAGE,
                Preference.TYPE_STRING_WITH_ACTION, "Application package (optional)", desc);
        PackageListManager pm = new PackageListManager();
        p.setComponentManager(pm);
        p.setPreferredContainerName(title);
        p.setAcceptEmptyValue(false);
        p.setMandatory(false);
        p.setHint(title,
                "<html><body><p>Specify the package name (optional). If the package name<br/>"
                + "is omitted the command will kill the last application installed through<br/>"
                + "the <code>com.tplan.adb.Install</code> script.<br/>"
                + "For details see the <a href=\""
                + ADB_SHELL_HELP_URL + "\"><code>am kill/force-stop</code> shell commands</a>.</p>"
                + "</body></html>");
        l.add(p);

        p = new Preference(PARAM_FORCE, Preference.TYPE_BOOLEAN, "Force the application stop", null);
        p.setPreferredContainerName(title);
        p.setDefaultValue(true);
        p.setHint(title,
                "<html><body><p>The optional <code>force</code> parameter switches between the forced stop "
                + "(<code>am force-stop</code>) and kill (<code>am kill</code>). The kill mode is a "
                + "softer variant which may or may not close the app depending on whether "
                + "<quote>it is safe to kill and that will not impact the user experience</quote>. "
                + "The default value is <b>on</b> (use <code>force-stop</code>).</p>"
                + "<p>For details see the <a href=\""
                + ADB_SHELL_HELP_URL + "\"><code>am kill</code> shell command</a>.</p>"
                + "</body></html>");
        l.add(p);
        p = new Preference(PARAM_LOG, Preference.TYPE_BOOLEAN, "Record the output to the execution log", null);
        p.setPreferredContainerName(title);
        p.setDefaultValue(false);
        p.setHint(title,
                "<html><body><p>The optional <code>log</code> parameter allows to write "
                + "the command output to the <a href=\"" + EXEC_LOG_HELP_URL + "\">execution log</a>. "
                + "It has the same effect as if you execute the <b>Log</b> command after the shell one as follows:</p><br>"
                + "<a href=\"" + TPR_SCRIPT_HELP_URL + "#log\">TPR test scripts:</a>"
                + "<ul><code>Log \"{" + VAR_SHELL_OUTPUT + "}\"</code></ul>"
                + "<a href=\"" + JAVA_SCRIPT_HELP_URL + "#log\">Java test scripts:</a>"
                + "<ul><code>log(getVariableAsString(\"" + VAR_SHELL_OUTPUT + "\"), java.util.logging.Level.INFO);</code></ul>"
                + "</body></html>");
        l.add(p);
        return l;
    }

    public String getShortDescription() {
        return "Kill an application identified by the package on the Android device connected over the ADB connection.";
    }
}
