/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.adb;

// Import the ADB loader to ensure that the libs get loaded
import com.tplan.adb.AdbLoader;

import com.android.ddmlib.IDevice;
import com.android.ddmlib.ShellCommandUnresponsiveException;
import com.tplan.robot.gui.MainFrame;
import com.tplan.robot.gui.components.StringComponent;
import com.tplan.robot.preferences.Preference;
import com.tplan.robot.preferences.PreferenceComponentManager;
import com.tplan.robot.remoteclient.RemoteDesktopClient;
import com.tplan.robot.remoteclient.adb.AdbClient;
import com.tplan.robot.scripting.DefaultJavaTestScript;
import com.tplan.robot.scripting.ParametrizedTestScript;
import com.tplan.robot.scripting.StopRequestException;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;

/**
 * Uninstall an application of the specified package from the Android device.
 */
public class Uninstall extends DefaultJavaTestScript implements Constants,
        ParametrizedTestScript, Runnable {

    // Parameter "package"
    public static final String PARAM_PACKAGE = "package";
    //
    // Member variables populated per execution
    protected IDevice device;
    protected String name;
    protected Exception exception;

    @Override
    public void test() {
        int exitCode = 1;
        try {
            // Load the command parameter of "package"
            // v2 update - if the package is not specified use the last installed 
            // one by the Install script
            String name = getVariableAsString(PARAM_PACKAGE);
            if (name == null) {
                name = (String) getContext().get(Install.VAR_PACKAGE);
                if (name == null && getContext().getMasterInterpret().isPartiallyExecuting() && Install.lastPackage != null) {
                    name = Install.lastPackage;
                }
            }
            if (name == null) {
                throw new IllegalArgumentException("No package name was specified and "
                        + "no application has been recently installed through the "+Install.class.getName()+" script!");                
            }

            // There must be a connected "Android over ADB" client
            RemoteDesktopClient client = getContext().getClient();
            if (client != null && client.isConnected() && client instanceof AdbClient) {
                AdbClient cl = (AdbClient) client;
                IDevice dev = (IDevice) cl.getIDevice();
                if (dev != null) {
                    try {
                        // Populate the member variables
                        this.device = dev;
                        this.name = name;
                        this.exception = null;

                        // Start the operation in a thread so that we can
                        // continue in case that the user stops the script.
                        // As the IDevice API doesn't allow to stop the operation
                        // we will just let the thread run on its own.
                        Thread t = new Thread(this);
                        t.start();

                        // Wait until the thread finishes. If the stop is requested
                        // cancel it.
                        while (t.isAlive()) {
                            if (getContext().getMasterInterpret().isStop()) {
                                t.interrupt();
                                throw new StopRequestException();
                            }
                            Thread.sleep(5);
                        }

                        // If the command executed in the thread threw an exception
                        // then rethrow it
                        if (exception != null) {
                            throw exception;
                        }

                        exitCode = 0;
                    } catch (Exception ex) {
                        if (ex instanceof ShellCommandUnresponsiveException || ex instanceof TimeoutException) {
                            log("Failed to uninstall application package '" + name + "' from the Android device: Operation timed out", Level.SEVERE);
                        } else if (ex instanceof StopRequestException) {
                            log("The script was stopped while uninstalling application package '" + name + "' from the Android device. The uninstallation result is not known.", Level.WARNING);
                        } else {
                            log("Failed to uninstall application package '" + name + "' from the Android device: Operation failed with " + ex.getClass().getName() + ": " + ex.getMessage(), Level.SEVERE);
                        }
                    }
                }
            } else {
                if (client == null || !client.isConnected()) {
                    log("Failed to uninstall application package '" + name + "' because no Android device is connected!", Level.SEVERE);
                } else {
                    log("Failed to uninstall application package '" + name + "' because the current connection ("
                            + client.getConnectString() + ") is not an Android Over ADB one!", Level.SEVERE);
                }
            }
        } catch (StopRequestException ex) {
            throw ex;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            getContext().setVariable("_EXIT_CODE", exitCode);
        }
    }

    public List<String> getParameters() {
        return Arrays.asList(new String[]{PARAM_PACKAGE});
    }

    public List getParameterValues(String parameterName) {
        return null;
    }

    public void setParameters(Map<String, String> paramsAndValues) {
    }

    public String getLongDescription() {
        return getShortDescription();
    }

    public String getShortDescription() {
        return "Uninstall an Android application from the device.";
    }

    public void run() {
        try {
            device.uninstallPackage(name);
        } catch (Exception ex) {
            this.exception = ex;
        }
    }

    public List<Preference> getVisualParameters() {
        List<Preference> l = new ArrayList<Preference>();
        String title = "ADB Uninstall";
        String desc = "<html><body><a href=\""
                + ADB_INSTALL_HELP_URL + "\">Uninstall</a> the specified application package from the Android device connected<br/>"
                + "over the "
                + "<a href=\"" + ANDROID_OVER_ADB_URL + "\">Android Over ADB</a> connection. If the package name<br/>"
                + "is omitted the command will uninstall the last application installed through<br/>"
                + "the <code>com.tplan.adb.Install</code> script.<br/>"
                + "<br/>For details see the <a href=\"" + PLUGIN_DOC_URL + "#" + getClass().getSimpleName().toLowerCase()
                + "\">" + getClass().getSimpleName() + " specification</a>."
                + "</body></html>";
        Preference p = new Preference(PARAM_PACKAGE,
                Preference.TYPE_STRING_WITH_ACTION, "Application package (optional)", desc);
        PackageListManager pm = new PackageListManager();
        p.setComponentManager(pm);
        p.setPreferredContainerName(title);
        p.setAcceptEmptyValue(false);
        p.setMandatory(false);
        p.setHint(title,
                "<html><body>"
                + "<p>Provide the application package to be uninstalled. "
                + "You may use the button on the right to browse the applications on the currectly "
                + "connected Android device. If you wish to uninstall the last application installed "
                + "using the <code>" + Install.class.getName() + "</code> script you may leave the package empty or specify it "
                + "using the <code>" + Install.VAR_PACKAGE + "</code> variable as follows:</p>"
                + "<ul><code>{" + Install.VAR_PACKAGE + "}</code></ul>"
                + "</body></html>");
        l.add(p);
        return l;
    }
}
